/***************************************************************************
 *   Copyright (C) 2003 by Peter Hess                                      *
 *   peter.hess@ph-home.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include <kglobal.h>
#include <klocale.h>
#include <kconfig.h>

#include "kwinfo.h"

kwInfo::kwInfo(QWidget *parent, const char *name)
 : QGrid(2, Qt::Horizontal, parent, "KWlanInfo-Info", Qt::WX11BypassWM | Qt::WStyle_StaysOnTop )
{
	setFrameStyle(QFrame::Box|QFrame::Plain);
	setLineWidth(1);
	setMargin(2);
	setSpacing(1);
	new QLabel("<b>"+i18n("Interface")+"</b>",this);
	(info_label.if_name = new QLabel(this,"If_name"))->setBackgroundMode(Qt::PaletteBase);
	new QLabel("<b>"+i18n("IP")+"</b>",this);
	(info_label.if_ip = new QLabel(this,"If_ip"))->setBackgroundMode(Qt::PaletteBase);
	new QLabel("<b>"+i18n("Signal")+"</b>",this);
	signalBar = new QMeterBar(-128,0,QMeterBar::qm_dbm,this,"");
	signalBar->setVisibleMinMax(true);
	new QLabel("<b>"+i18n("Noise")+"</b>",this);
	noiseBar = new QMeterBar(-128,0,QMeterBar::qm_dbm,this,"");
	noiseBar->setVisibleMinMax(true);
	new QLabel("<b>"+i18n("Quality")+"</b>",this);
	qualityBar = new QMeterBar(0,100,QMeterBar::qm_pro,this,"");
	qualityBar->setVisibleMinMax(true);
	new QLabel("<b>"+i18n("Nickname")+"</b>",this);
	(info_label.nick = new QLabel(this,"Nickname"))->setBackgroundMode(Qt::PaletteBase);
	new QLabel("<b>"+i18n("ESSID")+"</b>",this);
	(info_label.essid = new QLabel(this,"ESSID"))->setBackgroundMode(Qt::PaletteBase);
	new QLabel("<b>"+i18n("Mode")+"</b>",this);
	(info_label.mode = new QLabel(this,"Mode"))->setBackgroundMode(Qt::PaletteBase);
	new QLabel("<b>"+i18n("Access Point")+"</b>",this);
	(info_label.ap = new QLabel(this,"AccessPoint"))->setBackgroundMode(Qt::PaletteBase);
	new QLabel("<b>"+i18n("Freq[Channel]")+"</b>",this);
	(info_label.freq = new QLabel(this,"Freq"))->setBackgroundMode(Qt::PaletteBase);
	new QLabel("<b>"+i18n("Bit Rate")+"</b>",this);
	(info_label.rate = new QLabel(this,"Bitrate"))->setBackgroundMode(Qt::PaletteBase);
}


kwInfo::~kwInfo()
{
}

/*!
    \fn kwinfo::setBarColorsAndLevels(QColor lc, QColor mc, QColor hc, int ml, int hl)
 */
void kwInfo::setBarColorsAndLevels(QColor lc, QColor mc, QColor hc, int ml, int hl)
{
 	signalBar->setColors(lc,mc,hc);
	signalBar->setColorLevels(ml,hl);
	noiseBar->setColors(lc,mc,hc);
	noiseBar->setColorLevels(ml,hl);
	qualityBar->setColors(lc,mc,hc);
	qualityBar->setColorLevels(ml,hl);
}

/*!
    \fn kwinfo::setFontAndColor(QFont font, QColor color)
 */
void kwInfo::setFontAndColor(const QString &font, const QColor &color)
{
	signalBar->setFontFamily(font);
	signalBar->setFontColor(color);
	noiseBar->setFontFamily(font);
	noiseBar->setFontColor(color);
	qualityBar->setFontFamily(font);
	qualityBar->setFontColor(color);
}

#include "kwinfo.moc"
