/***************************************************************************
 *   Copyright (C) 2003 by Peter Hess                                      *
 *   peter.hess@ph-home.de                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <kglobal.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kmessagebox.h>
#include <qinputdialog.h>
#include <qstring.h>
#include <qlistbox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <kfontcombo.h>
#include <kcolorbutton.h>
#include <kaboutdata.h>
#include <qcolor.h>
#include <qtooltip.h>
#include "kwlaninfo.h"
#include "about.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>

KWlanInfo::KWlanInfo(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
    : KPanelApplet(configFile, type, actions, parent, name)
{
	errMsg = "";
	layout = new QVBoxLayout(this,0,1);
	layout->setAutoAdd(true);
	(errorInfo = new QLabel(this,"errorInfo"))->hide();
	barView = new kwBarView(this,"barView");
	info = new kwInfo();
 	ksConfig = config();
	readConfig();
	strncpy(ifr.ifr_name,if_dev.name,IFNAMSIZ);
	if ( (skt = socket(AF_INET, SOCK_STREAM, 0)) < 3 ) {
		errMsg="kwlaninfo error: "+i18n("Could not open socket!");
		KMessageBox::error(0,errMsg);
		sktOK = false;
	}
	else sktOK = true;
	if ( !checkForInterfaces() ) {
		errMsg = i18n("No interface found!");
		KMessageBox::error(0,errMsg);
	}
	if ( strlen(if_dev.name) == 0 )
		preferences();
	if ( sktOK )
		getSignals();
	timerEvent(0);
	timerID = startTimer(refresh*1000);
	setPanels();
	ttText = "<b>KWlanInfo</b><br>%1";
}


KWlanInfo::~KWlanInfo()
{
	::close(skt);
}


void KWlanInfo::about()
{
	aboutDlg* about = new aboutDlg(this,"About kwlaninfo");
	about->setAboutName(i18n("KWlanInfo version ")+ " " + VERSION);
	about->setAboutTab(i18n("This program displays some wlan information.<br>Currently it works only with wlan interfaces in <b>managed</b> mode.<br><center>&copy; 2003 - 2004 by Peter Hess</center>"));
	about->setLicenseTab(KAboutData("","","","",KAboutData::License_GPL).license());
	about->setAuthorTab("Peter Hess<br><a href='mailto:peter.hess@ph-home.de'>peter.hess@ph-home.de</a><br>");
	about->exec();
}


void KWlanInfo::help()
{
	KMessageBox::information(0, i18n("Sorry, no help text in this version"));
}


void KWlanInfo::preferences()
{
	prefDlg = new kwPrefs(this);
	prefDlg->iface->insertStringList(ifaces);
	prefDlg->iface->setSelected(0,true);
	prefDlg->refresh->setValue(refresh);
	if (viewMode == "condensed")
		prefDlg->viewModeCondensed->setChecked(true);
	else if (viewMode == "ultraCondensed")
		prefDlg->viewModeUltraCondensed->setChecked(true);
	else prefDlg->viewModeNormal->setChecked(true);
	if (infoMode == "onDoubleClick")
		prefDlg->infoDoubleClick->setChecked(true);
	else prefDlg->infoMouseOver->setChecked(true);
	prefDlg->noise->setChecked(noiseVisible);
	prefDlg->quality->setChecked(qualityVisible);
	prefDlg->barFont->setCurrentFont(ksConfig->readEntry("barFont","sans-serif"));
	prefDlg->barColorLow->setColor(ksConfig->readColorEntry("barColorLow",new QColor(QBrush::red)));
	prefDlg->barColorMiddle->setColor(ksConfig->readColorEntry("barColorMiddle",new QColor(QBrush::yellow)));
	prefDlg->barColorHigh->setColor(ksConfig->readColorEntry("barColorHigh",new QColor(QBrush::green)));
	prefDlg->barLevelMiddle->setValue(ksConfig->readEntry("barLevelMiddle","33").toInt());
	prefDlg->barLevelHigh->setValue(ksConfig->readEntry("barLevelHigh","66").toInt());
	connect(prefDlg->buttonApply,SIGNAL(released()),SLOT(applyConfig()));
	prefDlg->adjustSize();
	if (prefDlg->exec() == QDialog::Accepted)
		applyConfig();
	else configOK = false;
	delete(prefDlg);
	prefDlg = 0;
}

int KWlanInfo::widthForHeight(int height) const
{
	if ( viewMode == "condensed" ) return height;
	else if ( viewMode == "ultraCondensed" ) return height >> 1;
	else return height << 1;
}

int KWlanInfo::heightForWidth(int width) const
{
	if ( viewMode == "condensed"  ) return width;
	else if ( viewMode == "ultraCondensed" ) return width << 1;
	else return width >> 1;
}

extern "C"
{
    KPanelApplet* init( QWidget *parent, const QString& configFile)
    {
        KGlobal::locale()->insertCatalogue("kwlaninfo");
        return new KWlanInfo("kwlaninforc", KPanelApplet::Normal,
                             KPanelApplet::About | KPanelApplet::Help | KPanelApplet::Preferences,
                             parent, "KWlanInfo");
    }
}

/*!
    \fn KWlanInfo::*checkForInterfaces()
 */
bool KWlanInfo::checkForInterfaces()
{
	for (ifr.ifr_ifindex = 1; ifr.ifr_ifindex < 15; ifr.ifr_ifindex++) {
		strcpy(ifr.ifr_name,"");
		if ( ioctl(skt,SIOCGIFNAME,&ifr) == 0 ) {
			strncpy(iwr.ifr_ifrn.ifrn_name,ifr.ifr_name,IFNAMSIZ);
			iwr.u.data.pointer = (char *) &iw_stats;
			iwr.u.data.length = sizeof(iw_stats);
			if ( ioctl(skt,SIOCGIWSTATS,&iwr) == 0 ) {
				ifaces << ifr.ifr_name;
			}
		}
	}
	if (ifaces.count()>0)
		return true;
	return false;
}

/*!
    \fn KWlanInfo::getSignals()
 */
bool KWlanInfo::getSignals()
{
	strncpy(iwr.ifr_ifrn.ifrn_name, if_dev.name, IFNAMSIZ);
	iwr.u.data.pointer = (char *) &iw_stats;
	iwr.u.data.length = sizeof(iw_stats);
	if ( ioctl(skt, SIOCGIWSTATS, &iwr) == 0 ) {
		if_dev.signal = (__s8)iw_stats.qual.level;
		if_dev.noise = (__s8)iw_stats.qual.noise;
		if_dev.quality = iw_stats.qual.qual > 100?100:iw_stats.qual.qual;
		return true;
	}
	errMsg = i18n("Could not get statistics for ").append(if_dev.name);
	return false;
}

/*!
    \fn KWlanInfo::setPanels()
 */
void KWlanInfo::setPanels()
{
	barView->showBars(true,noiseVisible,qualityVisible);
}


/*!
    \fn KWlanInfo::setSignal(int sig)
 */
void KWlanInfo::setSignals()
{
	barView->updateValues(if_dev.signal,if_dev.noise,if_dev.quality);
	if (info->isVisible())
		setInfoText();
}

void KWlanInfo::timerEvent(QTimerEvent *)
{
	blockSignals(true);
	if ( !getSignals() ) {
		if ( errorInfo->isHidden() ) {
			errorInfo->setText("<center><b>kwlaninfo<br>!!! "+i18n("Error")+" !!!</b></center>");
			QToolTip::add(errorInfo,"<nobr><b><u>"+i18n("KWlanInfo Error Message").append("</u></b><br>"+errMsg+"</nobr>"));
			barView->hide();
			errorInfo->show();
		}
	}
	else {
		if ( errorInfo->isShown() ) {
			errorInfo->hide();
			QToolTip::remove(errorInfo);
			barView->show();
		}
		setSignals();
	}
	blockSignals(false);
}



/*!
    \fn KWlanInfo::readConfig()
 */
void KWlanInfo::readConfig()
{
	ksConfig->setGroup("Preferences");
	strcpy(if_dev.name,ksConfig->readEntry("interface",""));
	strcpy(if_dev.type,ksConfig->readEntry("type",""));
	refresh = ksConfig->readEntry("refresh","1").toInt();
	viewMode = ksConfig->readEntry("viewMode","normal");
	infoMode = ksConfig->readEntry("infoMode","onMouseOver");
	noiseVisible = ksConfig->readEntry("noisebar","yes")=="yes"?true:false;
	qualityVisible = ksConfig->readEntry("qualitybar","yes")=="yes"?true:false;
	QString f = ksConfig->readEntry("barFont","sans-serif");
	QColor fc = ksConfig->readColorEntry("barFontColor",new QColor(QBrush::black));
	QColor lc = ksConfig->readColorEntry("barColorLow",new QColor(QBrush::red));
	QColor mc = ksConfig->readColorEntry("barColorMiddle",new QColor(QBrush::yellow));
	QColor hc = ksConfig->readColorEntry("barColorHigh",new QColor(QBrush::green));
	int ml = ksConfig->readEntry("barLevelMiddle","33").toInt();
	int hl = ksConfig->readEntry("barLevelHigh","66").toInt();
	barView->setFontAndColor(f,fc);
	barView->setBarColorsAndLevels(lc,mc,hc,ml,hl);
	info->setFontAndColor(f,fc);
	info->setBarColorsAndLevels(lc,mc,hc,ml,hl);
}

/*!
    \fn KWlanInfo::writeConfig()
*/
bool KWlanInfo::writeConfig()
{
	if (!prefDlg) return false;
	ksConfig->setGroup("Preferences");
	ksConfig->writeEntry("interface",prefDlg->iface->currentText());
	ksConfig->writeEntry("refresh",prefDlg->refresh->value());
	if (prefDlg->viewModeCondensed->isChecked())
		ksConfig->writeEntry("viewMode","condensed");
	else if (prefDlg->viewModeUltraCondensed->isChecked())
		ksConfig->writeEntry("viewMode","ultraCondensed");
	else ksConfig->writeEntry("viewMode","normal");
	if (prefDlg->infoDoubleClick->isChecked())
		ksConfig->writeEntry("infoMode","onDoubleClick");
	else ksConfig->writeEntry("infoMode","onMouseOver");
	// bar preferences visible
	ksConfig->writeEntry("noisebar",prefDlg->noise->isChecked()?"yes":"no");
	ksConfig->writeEntry("qualitybar",prefDlg->quality->isChecked()?"yes":"no");
	// bar preferences font
	ksConfig->writeEntry("barFont",prefDlg->barFont->currentText());
	ksConfig->writeEntry("barFontColor",prefDlg->barFontColor->color());
	// bar preferences level colors
	ksConfig->writeEntry("barColorLow",prefDlg->barColorLow->color());
	ksConfig->writeEntry("barColorMiddle",prefDlg->barColorMiddle->color());
	ksConfig->writeEntry("barColorHigh",prefDlg->barColorHigh->color());
	// bar change level of colors
	ksConfig->writeEntry("barLevelMiddle",prefDlg->barLevelMiddle->value());
	ksConfig->writeEntry("barLevelHigh",prefDlg->barLevelHigh->value());
	ksConfig->sync();
	return true;
}

/*!
    \fn KWlanInfo::applyConfig()
 */
void KWlanInfo::applyConfig()
{
	if (writeConfig()) {
		readConfig();
		killTimer(timerID);
		timerEvent(0);
		timerID = startTimer(refresh*1000);
		updateLayout();
		setPanels();
	}
}


/*!
    \fn KWlanInfo::enterEvent(QEvent* )
 */
void KWlanInfo::enterEvent(QEvent* )
{
	if ( (infoMode != "onMouseOver") || !sktOK ) return;
	showInfo();
	setActiveWindow();
}


/*!
    \fn KWlanInfo::leaveEvent(QEvent*)
 */
void KWlanInfo::leaveEvent(QEvent*)
{
	if ( info->isShown() ) info->hide();
}


/*!
    \fn KWlanInfo::read_iwconfig()
 */
bool KWlanInfo::read_iwconfig()
{
	strcpy(ifr.ifr_name,if_dev.name);
	if ( ioctl(skt,SIOCGIFADDR,&ifr) == 0 )
		sprintf(if_dev.ip,"%d.%d.%d.%d",
		          ifr.ifr_addr.sa_data[2]&0xff,ifr.ifr_addr.sa_data[3]&0xff,
			  ifr.ifr_addr.sa_data[4]&0xff,ifr.ifr_addr.sa_data[5]&0xff);
	else
		strcpy(if_dev.ip,"???");
	char buf[IW_ESSID_MAX_SIZE+1];
	const char * iw_modes[] = {"Auto","Ad-Hoc","Managed","Master","Repeater","Secondary","Monitor"};
	memset(&iwr, 0, sizeof(iwr));
	strcpy(iwr.ifr_ifrn.ifrn_name,if_dev.name);
	iwr.u.data.pointer = (caddr_t)buf;
	iwr.u.data.length = sizeof(buf);
	memset(&buf, 0, sizeof(buf));
	if ( ioctl(skt,SIOCGIWESSID,&iwr) == 0 )
		strcpy(if_dev.essid,buf);
	else
		strcpy(if_dev.essid,"???");
	iwr.u.data.pointer = (caddr_t)buf;
	iwr.u.data.length = sizeof(buf);
	memset(&buf, 0, sizeof(buf));
	if ( ioctl(skt,SIOCGIWNICKN,&iwr) == 0 )
		strcpy(if_dev.nick,buf);
	else
		strcpy(if_dev.nick,"???");
	if ( ioctl(skt,SIOCGIWMODE,&iwr) == 0 )
		if ( iwr.u.mode < 7 )
			strcpy(if_dev.mode,iw_modes[iwr.u.mode]);
	if ( ioctl(skt,SIOCGIWAP,&iwr) == 0 )
		sprintf(if_dev.ap,"%02X:%02X:%02X:%02X:%02X:%02X",
		          iwr.u.addr.sa_data[0]&0xff,iwr.u.addr.sa_data[1]&0xff,iwr.u.addr.sa_data[2]&0xff,
			  iwr.u.addr.sa_data[3]&0xff,iwr.u.addr.sa_data[4]&0xff,iwr.u.addr.sa_data[5]&0xff);
	if ( ioctl(skt,SIOCGIWRATE,&iwr) == 0 )
		sprintf(if_dev.bitrate,"%d Mbs",iwr.u.bitrate.value/1000000);
	if ( ioctl(skt,SIOCGIWFREQ,&iwr) == 0 ) {
		if ( iwr.u.freq.e > 0 ) {
			if_dev.freq = iwr.u.freq.m*exp10(iwr.u.freq.e-9);
		} else {
			/* Workaround for drivers that return a channel number rather than a frequency */
			if_dev.freq = (float)(2407 + 5 * iwr.u.freq.m) / 1000;
		}
	}
       return true;
 }


/*!
    \fn KWlanInfo::setInfoText()
 */
void KWlanInfo::setInfoText(bool onlySignals)
{
	info->signalBar->setSignal(if_dev.signal);
	info->noiseBar->setSignal(if_dev.noise);
	info->qualityBar->setSignal(if_dev.quality);
	if ( !onlySignals ) {
		read_iwconfig();
		info->info_label.if_name->setText(if_dev.name);
		info->info_label.if_ip->setText(if_dev.ip);
		info->info_label.nick->setText(if_dev.nick);
		info->info_label.essid->setText(if_dev.essid);
		info->info_label.mode->setText(if_dev.mode);
		info->info_label.ap->setText(if_dev.ap);
		info->info_label.freq->setText(QCString().setNum(if_dev.freq)+"GHz["+
			QCString().setNum((int)(if_dev.freq*1000 + 0.5 - 2407)/5)+"]");
		info->info_label.rate->setText(if_dev.bitrate);
	}
	info->adjustSize();
}

/*!
    \fn KWlanInfo::mouseDoubleClickEvent(QMouseEvent *e)
 */
void KWlanInfo::mouseDoubleClickEvent(QMouseEvent *)
{
	if ( (infoMode != "onDoubleClick") || !sktOK ) return;
	showInfo();
}

/*!
    \fn KWlanInfo::showInfo()
 */
void KWlanInfo::showInfo()
{
	setInfoText(false);
	QPoint p = mapToGlobal (QPoint(-info->width()+width(),-info->height()-4));
	if ( position() == pTop ) p.setY(height()+4);
	if ( position() == pLeft ) p.setX(width()+4);
	if ( position() == pRight) p.setX(p.x()-width()-4);
	if ( p.x() < 0 ) p.setX(0);
	if ( p.y() < 0 ) p.setY(0);
	info->move(p);
	info->show();
}
