/***************************************************************************
 *   Copyright (C) 2003 by Peter Hess                                      *
 *   peter.hess@ph-home.de                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef KWlanInfo_H
#define KWlanInfo_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kpanelapplet.h>
#include <qstring.h>
#include <kconfig.h>
#include <qfile.h>
#include <qstringlist.h>
#include <qlabel.h>
#include <qgrid.h>
#include <qlayout.h>
#include <qstringlist.h>

#include "kwprefs.h"
#include "kwbarview.h"
#include "qmeterbar.h"
#include "kwinfo.h"
#include "kwprofile.h"

#include <sys/ioctl.h>
#include <sys/socket.h>
#include <linux/types.h>
#include <linux/if.h>
#include <linux/wireless.h>
#undef VERSION
#define VERSION "0.9.5"

class KWlanInfo : public KPanelApplet
{
    Q_OBJECT

public:
    /**
     * Construct a @ref KPanelApplet just like any other widget.
     *
     * @param configFile The configFile handed over in the factory function.
     * @param Type The applet @ref type().
     * @param actions Standard RMB menu actions supported by the applet (see @ref action() ).
     * @param parent The pointer to the parent widget handed over in the factory function.
     * @param name A Qt object name for your applet.
     **/
    KWlanInfo(const QString& configFile, Type t = Normal, int actions = 0,
        QWidget *parent = 0, const char *name = 0);
    /** destructor */
    ~KWlanInfo();

    /**
     * Retrieve a suggested width for a given height.
     *
     * Every applet should reimplement this function.
     *
     * Depending on the panel orientation the height (horizontal panel) or the
     * width (vertical panel) of the applets is fixed.
     * The exact values of the fixed size component depend on the panel size.
     *
     * On a horizontal panel the applet height is fixed, the panel will
     * call @ref widthForHeight(int height) with @p height
     * equal to 'the fixed applet height'
     * when laying out the applets.
     *
     * The applet can now choose the other size component (width)
     * based on the given height.
     *
     * The width you return is granted.
     **/
    virtual int widthForHeight(int height) const;
    /**
     * @return A suggested height for a given width.
     *
     * Every applet should reimplement this function.
     *
     * Depending on the panel orientation the height (horizontal panel) or the
     * width (vertical panel) of the applets is fixed.
     * The exact values of the fixed size component depend on the panel size.
     *
     * On a vertical panel the applet width is fixed, the panel will
     * call @ref heightForWidth(int width) with @p width
     * equal to 'the fixed applet width'
     * when laying out the applets.
     *
     * The applet can now choose the other size component (height)
     * based on the given width.
     *
     * The height you return is granted.
     **/
    virtual int heightForWidth(int width) const;
    /**
     * Is called when the user selects "About" from the applets RMB menu.
     * Reimplement this function to launch a about dialog.
     *
     * Note that this is called only when your applet supports the About action.
     * See @ref Action and @ref KPanelApplet().
     **/
    virtual void about();
    /**
     * Is called when the user selects "Help" from the applets RMB menu.
     * Reimplement this function to launch a manual or help page.
     *
     * Note that this is called only when your applet supports the Help action.
     * See @ref Action and @ref KPanelApplet().
     **/
    virtual void help();
    /**
     * Is called when the user selects "Preferences" from the applets RMB menu.
     * Reimplement this function to launch a preferences dialog or kcontrol module.
     *
     * Note that this is called only when your applet supports the preferences action.
     * See @ref Action and @ref KPanelApplet().
     **/
    virtual void preferences();

public:
    bool checkForInterfaces();
    bool getSignals();
//    void read_ifconfig();

private:
    void showInfo();
    KConfig *ksConfig;
    kwPrefs* prefDlg;
    QWidget *mainView;
    QStringList ifaces;
    int timerID;
    QLabel *errorInfo;
    bool sktOK;
    bool noiseVisible, qualityVisible;
    int refresh;
    bool configOK;
    QString errMsg;
    QVBoxLayout *layout;
    QCString viewMode, infoMode;
    QString ttText;
//    QList <kwProfile> profiles;

   struct sIfDev {
    	char ip[16];
	char name[IFNAMSIZ];
	char type[30];
	char nwid[30];
	char essid[IW_ESSID_MAX_SIZE+1];
	char nick[IW_ESSID_MAX_SIZE+1];
	char mode[30];
	char ap[18];
	float freq;
	char bitrate[10];
	float rate;
	int signal;
	int noise;
	int quality;
	int status;
    } if_dev;

    struct ifreq ifr;    
    struct iwreq iwr;
    struct iw_statistics iw_stats;
    int skt; 
    
 protected:
    void timerEvent(QTimerEvent *);
    void readConfig();
    bool writeConfig();
    bool read_iwconfig();
    void setInfoText(bool onlySignals = true);
    void setPanels();


public slots:
    void setSignals();
    void applyConfig();
protected:
    kwInfo *info;
    kwBarView *barView;
    QLabel *labelSignals, *labelConfig;
    virtual void enterEvent(QEvent*);
    virtual void leaveEvent(QEvent*);
    virtual void mouseDoubleClickEvent(QMouseEvent *);
   };

#endif
